#!/bin/bash
#
#------------------------------------------
# This is the rsh/ssh version of forallbg
#------------------------------------------
#
# Setting up ssh remote execution:
#
# 1. Execute on one of the hosts to create a RSA key:
#	ssh-keygen -t rsa -f ${HOME}/.ssh/id_rsa -N ""
#	cp -af ${HOME}/.ssh/id_rsa.pub ${HOME}/.ssh/authorized_keys
#
# 2. If $HOME is not shared, cp '${HOME}/.ssh/id_rsa*' and
#    '${HOME}/.ssh/authorized_keys' to all hosts.
#
# 3. From one of the host, execute the following command:
#	for h in list_of_hosts; do ./yes.exp; done
#    where 'yes.exp' contains the following lines:
#	#!/usr/bin/expect -f
#	set timeout -1
#	match_max 100000
#	spawn ssh $argv true
#	expect "(yes/no)? "
#	send -- "yes\r"
#	expect eof
#

echo 1>&2 "Rsh version of forallbg, v0.1"

. /etc/SciCluster.inc
pwd=`pwd`
hosts=`${SCRIPTS_DIR}/cnodes`

n=0
tmpdir=/tmp/forallbg-`hostname`.$$
rm -rf $tmpdir
mkdir $tmpdir
trap "rm -rf $tmpdir" 0 1 2 3 15
for h in $hosts; do
    echo 1>&2 -n "$h "
    rsh -n $h "cd $pwd; $*" </dev/null >${tmpdir}/$h 2>&1 &
    n=$(($n+1))
    [ $(($n%8)) -eq 0 ] && echo 1>&2 ""
    [ $(($n%128)) -eq 0 ] && echo 1>&2 "waiting..." && wait && sleep 1
done
[ $(($n%8)) -ne 0 ] && echo 1>&2 ""
echo 1>&2 "waiting..."
wait

echo 1>&2
echo 1>&2 Outputs:
n=0
for h in $hosts; do
    echo ======================================== $h
    cat ${tmpdir}/$h
    n=$(($n+1))
done
echo ========================================

echo 1>&2 "Command executed on $n host(s)."
