#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
  #include <signal.h>
#include <fcntl.h>
#include <sys/param.h>
#include <sys/types.h>
#include <sys/stat.h>

const char *loadinfo_path="/usr/local/loadinfo";
const char *nodes_path="/usr/local/nodes";
int cno1, cno2, sno1, sno2, interval, down_time;

void update(int dummy);

int main(int argc, char *argv[])
{
    if (argc!=5) {
	fprintf(stderr, "Usage: check_nodes cno1 cno2 check_interval down_time\n");
	exit(1);
    }
    
    cno1=atoi(argv[1]);
    cno2=atoi(argv[2]);
    interval=atoi(argv[3]);
    down_time=atoi(argv[4]);

    if ( interval<=0 || down_time<=0 ) {
	fprintf(stderr, "Invalid update interval or down time.\n");
	exit(2);
    }

    if (chdir(loadinfo_path)) {
	fprintf(stderr, "Cannot change to directory %s\n", loadinfo_path);
	exit(3);
    }

#ifndef DEBUG
    if (getuid()) {
	fprintf(stderr, "check_nodes: not super user\n");
	exit(4);
    }
    daemon(1, 0);
    signal(SIGALRM, update);
    update(0);
#endif


    for (;;) {
	sleep(interval);
	update(0);
    }
}

void check_node(const char *node)
{
    struct stat st;
    char path[MAXPATHLEN];
    char loadinfo[MAXPATHLEN];
    time_t currtime;
    int fd;

    sprintf(path, "%s/%s", nodes_path, node);
    sprintf(loadinfo, "%s/%s", loadinfo_path, node);
    if ( stat(loadinfo, &st) ) {
	unlink(path);
	return;
    }

    if ( time(&currtime) - st.st_ctime > down_time ) {
	unlink(path);
    } else {
	if ((fd=open(path, O_WRONLY | O_CREAT, 0644))>0) close(fd);
    }

#ifdef DEBUG
    printf("%s is unchanged for %d seconds\n", node, currtime-st.st_ctime);
#endif
}

void update(int dummy)
{
    int i;
    char node[16];

    signal(SIGALRM, update);
    for (i=cno1; i<=cno2; i++) {
	sprintf(node, "c%d", i);
	check_node(node);
    }
    (void) alarm(interval);
}
