#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include <fcntl.h>
#include <sys/param.h>
#include <asm/param.h>
#include <sys/types.h>
#include <sys/stat.h>

/*#include "daemon.h"*/

int interval;	/* update every interval seconds */

const char *loadinfo_path="/usr/local/loadinfo";
char path[MAXPATHLEN];
char buffer[MAXHOSTNAMELEN];

void update(int dummy);

int main(int argc, char *argv[])
{
    char *cp;

    if (argc!=2 ) {
	fprintf(stderr, "Usage: loadd update_interval \n");
	exit(1);
    }

    interval=atoi(argv[1]);
    if (interval<=0) {
	fprintf(stderr, "Invalid update interval.\n");
	exit(2);
    }

    if (getuid()) {
	fprintf(stderr, "loadd: not super user\n");
	exit(3);
    }

    if (gethostname(buffer, sizeof (buffer) - 1) < 0) {
	fprintf(stderr, "Cannot get hostname.\n");
	exit(1);
    }
    if ((cp = index(buffer, '.')) != NULL) *cp = '\0';
    sprintf(path, "%s/%s", loadinfo_path, buffer);
    
    daemon(0, 0);

    signal(SIGALRM, update);
    update(0);

    for (;;) {
	sleep(interval);
	update(0);
    }
}

void update(int dummy)
{
    static double loadavg=0, uptime=0;
    FILE *fp;
    int fd, size;
    struct stat st;

    signal(SIGALRM, update);

    if ( (fp=fopen("/proc/loadavg", "r"))!=NULL ) {
	fscanf(fp, "%lf", &loadavg);
	fclose(fp);
    }

    if ( (fp=fopen("/proc/uptime", "r"))!=NULL ) {
	fscanf(fp, "%lf", &uptime);	/* "up_time  idle_time" */
	uptime/=3600;			/* convert to hours */
	fclose(fp);
    }


    fd = open(path, O_WRONLY | O_CREAT, 0644);
    if ( fd > 0) {
	sprintf(buffer, "%0.2lf %0.2lf\n", loadavg, uptime);
	size=strlen(buffer);
	write(fd, buffer, size);
	if (fstat(fd, &st) < 0 || st.st_size > size)
	ftruncate(fd, size);
	close(fd);
    }

    (void) alarm(interval);
}
